@@ -1,6 +1,7 @@
 import numpy as np
 from ruleskit import Activation
 import pytest
+from bitarray import bitarray
 
 Activation.FORCE_STAT = True
 
@@ -12,23 +13,23 @@ Activation.FORCE_STAT = True
             np.array([1, 0, 1]),
             "1,1,2,3",
             np.array([1, 1, 2, 3]),
-            5,
+            bitarray([1, 0, 1]),
             3,
         ),
         (
             np.array([0, 1, 0, 1]),
             "0,1,2,3,4",
             np.array([0, 1, 2, 3, 4]),
-            5,
+            bitarray("0101"),
             4,
         ),
     ],
 )
 def test_init(vector, cs, ca, i, n):
     res = Activation(vector)
-    np.testing.assert_equal(res.as_int, i)
-    np.testing.assert_equal(res.as_compressed_array, ca)
+    np.testing.assert_equal(res.as_bitarray, i)
     np.testing.assert_equal(res.as_compressed_str, cs)
+    np.testing.assert_equal(res.as_compressed_array, ca)
     np.testing.assert_equal(res.length, n)
 
 
@@ -107,24 +108,25 @@ def test_and(vector1, vector2, and_vector):
 def test_add(vector1, vector2, add_vector):
     act1 = Activation(vector1)
     act2 = Activation(vector2)
-    np.testing.assert_equal((act1 + act2).raw, add_vector)
+    added = (act1 + act2).raw
+    np.testing.assert_equal(added, add_vector)
 
 
 def test_sizes():
     act = Activation(np.array([1, 0, 1]))
     assert act.sizeof_raw > 0
-    assert act.sizeof_integer > 0
+    assert act.sizeof_bitarray > 0
     assert act.sizeof_compressed_array > 0
     assert act.sizeof_compressed_str > 0
 
 
 def test_times():
     act = Activation(np.array([1, 0, 1]))
-    assert act.time_compress > -1
-    assert act.time_decompress > -1
-    assert act.time_conversions_to_int > -1
-    assert act.time_conversions_from_int > -1
-    assert act.decompressions > 0
-    assert act.compressions > 0
-    assert act.conversions_to_int > 0
-    assert act.conversions_from_int > 0
+    assert act.time_raw_to_compressed > -1
+    assert act.time_compressed_to_raw > -1
+    assert act.time_raw_to_bitarray > -1
+    assert act.time_bitarray_to_raw > -1
+    assert act.n_compressed_to_raw > 0
+    assert act.n_raw_to_compressed > 0
+    assert act.n_raw_to_bitarray > 0
+    assert act.n_bitarray_to_raw > 0
