@@ -9,10 +9,14 @@ import pytest
     "rule1, rule2, rs",
     [
         (
-                Rule(HyperrectangleCondition([0], bmins=[1], bmaxs=[2])),
-                Rule(HyperrectangleCondition([1], bmins=[4], bmaxs=[5])),
-                RuleSet([Rule(HyperrectangleCondition([0], bmins=[1], bmaxs=[2])),
-                         Rule(HyperrectangleCondition([1], bmins=[4], bmaxs=[5]))]),
+            Rule(HyperrectangleCondition([0], bmins=[1], bmaxs=[2])),
+            Rule(HyperrectangleCondition([1], bmins=[4], bmaxs=[5])),
+            RuleSet(
+                [
+                    Rule(HyperrectangleCondition([0], bmins=[1], bmaxs=[2])),
+                    Rule(HyperrectangleCondition([1], bmins=[4], bmaxs=[5])),
+                ]
+            ),
         ),
     ],
 )
@@ -26,11 +30,13 @@ def test_add(rule1, rule2, rs):
     "xs, y, rule_list, coverage",
     [
         (
-                np.array([[5, 3], [3, 4], [2, np.nan]]),
-                np.array([1, 3, 2]),
-                [Rule(HyperrectangleCondition([0], bmins=[1], bmaxs=[2])),
-                 Rule(HyperrectangleCondition([1], bmins=[4], bmaxs=[5]))],
-                2/3,
+            np.array([[5, 3], [3, 4], [2, np.nan]]),
+            np.array([1, 3, 2]),
+            [
+                Rule(HyperrectangleCondition([0], bmins=[1], bmaxs=[2])),
+                Rule(HyperrectangleCondition([1], bmins=[4], bmaxs=[5])),
+            ],
+            2 / 3,
         ),
     ],
 )
